@@ -1,30 +1,34 @@
-# Copyright (c) 2009, 2016, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2020, Oracle and/or its affiliates.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2 of the
-# License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms, as
+# designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+# This program is distributed in the hope that it will be useful,  but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+# the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301  USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
-from mforms import newTreeNodeView, newButton, newBox, newSelector, newCheckBox, newLabel, Utilities
+from mforms import newTreeView, newButton, newBox, newSelector, newCheckBox, newLabel, Utilities
 import mforms
 import grt
 
-from workbench.db_utils import escape_sql_string
+from workbench.db_utils import escape_sql_string, QueryError
 
 from functools import partial
 
 from wb_common import dprint_ex
-from wb_admin_utils import weakcb, WbAdminBaseTab
+from wb_admin_utils import weakcb, WbAdminTabBase, WbAdminValidationBase, WbAdminValidationConnection
 import json
 
 from workbench.log import log_error
@@ -45,7 +49,7 @@ class WBThreadStack(mforms.Form):
 
         splitter = mforms.newSplitter(True, False)
 
-        self.tree = mforms.newTreeNodeView(mforms.TreeDefault)
+        self.tree = mforms.newTreeView(mforms.TreeDefault)
         self.tree.add_column(mforms.IntegerColumnType, "Event Id", 50, False)
         self.tree.add_column(mforms.StringColumnType, "Event info", 200, False)
         self.tree.add_column(mforms.StringColumnType, "Type", 100, False)
@@ -82,7 +86,7 @@ class WBThreadStack(mforms.Form):
         self.ok = newButton()
         self.ok.set_text("Close")
         self.ok.add_clicked_callback(self.close_form)
-        bbox.add_end(self.ok, False, False)
+        bbox.add_end(self.ok, False, True)
         vbox.add_end(bbox, False, True)
 
         self.set_size(800, 600)
@@ -110,7 +114,7 @@ class WBThreadStack(mforms.Form):
             if data:
                 data = data.replace("\0", "") # workaround for server bug
                 return self.parse_data(json.loads(data))
-        except Exception, e:
+        except Exception as e:
             import traceback
             #open("/tmp/data.js", "w+").write(data)
             log_error("Exception during sys.ps_thread_stack(%d, %s):\n%s\n" % (self.thread_id, "TRUE" if self.enable_debug_info else "FALSE", traceback.format_exc()))
@@ -185,21 +189,22 @@ class ConnectionDetailsPanel(mforms.Table):
         self.set_row_count(16)
 
         self.labels = {}
-        self.make_line("Processlist Id:", "PROCESSLIST_ID")
-        self.make_line("Thread Id:", "THREAD_ID")
-        self.make_line("Name:", "NAME")
-        self.make_line("Type:", "TYPE")
-        self.make_line("User:", "PROCESSLIST_USER")
-        self.make_line("Host:", "PROCESSLIST_HOST")
-        self.make_line("Schema:", "PROCESSLIST_DB")
-        self.make_line("Command:", "PROCESSLIST_COMMAND")
-        self.make_line("Time:", "PROCESSLIST_TIME")
-        self.make_line("State:", "PROCESSLIST_STATE")
-        self.make_line("Role:", "ROLE")
-        self.make_line("Instrumented:", "INSTRUMENTED")
-        self.make_line("Parent Thread Id:", "PARENT_THREAD_ID")
+        self.make_line("Processlist Id:", "PROCESSLIST_ID", "Process List")
+        self.make_line("Thread Id:", "THREAD_ID", "Thread Identifier")
+        self.make_line("Name:", "NAME", "Name")
+        self.make_line("Type:", "TYPE", "Type")
+        self.make_line("User:", "PROCESSLIST_USER", "Process List User")
+        self.make_line("Host:", "PROCESSLIST_HOST", "Process List Host")
+        self.make_line("Schema:", "PROCESSLIST_DB", "Process List Database")
+        self.make_line("Command:", "PROCESSLIST_COMMAND", "Process List Command")
+        self.make_line("Time:", "PROCESSLIST_TIME", "Process List Time")
+        self.make_line("State:", "PROCESSLIST_STATE", "Process List State")
+        self.make_line("Role:", "ROLE", "Role")
+        self.make_line("Instrumented:", "INSTRUMENTED", "Instrumented")
+        self.make_line("Parent Thread Id:", "PARENT_THREAD_ID", "Parent Thread Identified")
 
         l = mforms.newLabel("Info:")
+        l.set_name("Info")
         l.set_style(mforms.BoldStyle)
         self.add(l, 0, 1, 13, 14, mforms.HFillFlag|mforms.HExpandFlag)
         self.info = mforms.newCodeEditor()
@@ -221,9 +226,10 @@ class ConnectionDetailsPanel(mforms.Table):
 
 
 
-    def make_line(self, caption, name):
+    def make_line(self, caption, name, accessibilityName):
         i = len(self.labels)
         l = mforms.newLabel(caption)
+        l.set_name(accessibilityName)
         l.set_text_align(mforms.MiddleLeft)
         l.set_style(mforms.BoldStyle)
         self.add(l, 0, 1, i, i+1, mforms.HFillFlag|mforms.HExpandFlag)
@@ -253,30 +259,54 @@ class ConnectionDetailsPanel(mforms.Table):
             self.explain.set_enabled(True if node and node.get_tag() else False)
 
 
-class WbAdminConnections(WbAdminBaseTab):
-    ui_created = False
+class WbAdminValidationPermissions(WbAdminValidationBase):
+    def __init__(self, ctrl_be):
+        super().__init__()
+        self._ctrl_be = ctrl_be
+        
+    def validate(self):
+        try:
+            self._ctrl_be.exec_query("SELECT COUNT(*) FROM performance_schema.threads")
+        except QueryError as e:
+            import traceback
+            log_error("QueryError in Admin for Client Connections:\n%s\n\n%s\n" % (e, traceback.format_exc()))
+            if e.error == 1142:
+                self.set_error_message("The account you are currently using does not have sufficient privileges to view the client connections.")
+            else:
+                self.set_error_message("There was a problem opening the Client Connections. Please check the error log for more details.")
+            return False
+        except Exception as e:
+            import traceback
+            log_error("Exception in Admin for Client Connections:\n%s\n\n%s\n" % (e, traceback.format_exc()))
+            self.set_error_message("There was a problem opening the Client Connections. Please check the error log for more details.")
+            return False
+        return True
+
+class WbAdminConnections(WbAdminTabBase):
+    #ui_created = False
     serial = 0
 
     @classmethod
     def wba_register(cls, admin_context):
-        admin_context.register_page(cls, "wba_management", "Client Connections", False)
+        admin_context.register_page(cls, "Management", "Client Connections", "Client Connections", False)
 
     @classmethod
     def identifier(cls):
         return "admin_connections"
 
     def __init__(self, ctrl_be, instance_info, main_view):
-        mforms.Box.__init__(self, False)
+        WbAdminTabBase.__init__(self, ctrl_be, instance_info, main_view)
+        
+        self.add_validation(WbAdminValidationConnection(ctrl_be))
+        self.add_validation(WbAdminValidationPermissions(ctrl_be))
+        
+        self.set_standard_header("title_connections.png", self.instance_info.name, "Client Connections")
+        
         self.set_managed()
         self.set_release_on_add()
-        self.set_padding(12)
-        self.set_spacing(15)
-        self.instance_info = instance_info
-        self.ctrl_be = ctrl_be
-        self.page_active = False
-        self.main_view = main_view
         self._new_processlist = self.check_if_ps_available()
         self._refresh_timeout = None
+        self.warning = None
         
         if self.new_processlist():
             self.columns = [("PROCESSLIST_ID", mforms.LongIntegerColumnType, "Id", 50),
@@ -321,12 +351,12 @@ class WbAdminConnections(WbAdminBaseTab):
                             ]
             self.long_int_columns = [0, 5]
             self.info_column = 7
+            
 
     def create_ui(self):
         dprint_ex(4, "Enter")
-        self.suspend_layout()
-
-        self.create_basic_ui("title_connections.png", "Client Connections")
+        
+        uiBox = mforms.newBox(False)
 
         if self.new_processlist():
             widths = grt.root.wb.state.get("wb.admin:ConnectionListColumnWidthsPS", None)
@@ -339,9 +369,10 @@ class WbAdminConnections(WbAdminBaseTab):
 
         self.connection_box = mforms.newBox(True)
         self.connection_box.set_spacing(8)
-        self.connection_list = newTreeNodeView(mforms.TreeDefault|mforms.TreeFlatList|mforms.TreeAltRowColors)
+        self.connection_list = newTreeView(mforms.TreeDefault|mforms.TreeFlatList|mforms.TreeAltRowColors)
         self.connection_list.set_selection_mode(mforms.TreeSelectMultiple)
         self.connection_list.add_column_resized_callback(self.column_resized)
+        self.connection_list.set_name("Connection List")
         for i, (field, type, caption, width) in enumerate(self.columns):
             if column_widths and i < len(column_widths):
                 width = column_widths[i]
@@ -359,49 +390,48 @@ class WbAdminConnections(WbAdminBaseTab):
         info_table.set_column_count(5)
         info_table.set_row_spacing(4)
         info_table.set_column_spacing(20)
+        info_table.set_name("Counters")
 
-        info_table.add(self.create_labeled_info("Threads Connected:", "lbl_Threads_connected"),                     0, 1, 0, 1, mforms.HFillFlag)
-        info_table.add(self.create_labeled_info("Threads Running:", "lbl_Threads_running"),                          1, 2, 0, 1, mforms.HFillFlag)
-        info_table.add(self.create_labeled_info("Threads Created:", "lbl_Threads_created"),                         2, 3, 0, 1, mforms.HFillFlag)
-        info_table.add(self.create_labeled_info("Threads Cached:", "lbl_Threads_cached"),                           3, 4, 0, 1, mforms.HFillFlag)
-        info_table.add(self.create_labeled_info("Rejected (over limit):", "lbl_Connection_errors_max_connections"), 4, 5, 0, 1, mforms.HFillFlag)
+        info_table.add(self.create_labeled_info("Threads Connected:", "Threads Connected", "lbl_Threads_connected"),                     0, 1, 0, 1, mforms.HFillFlag | mforms.VFillFlag)
+        info_table.add(self.create_labeled_info("Threads Running:", "Threads Running", "lbl_Threads_running"),                          1, 2, 0, 1, mforms.HFillFlag | mforms.VFillFlag)
+        info_table.add(self.create_labeled_info("Threads Created:", "Threads Created", "lbl_Threads_created"),                         2, 3, 0, 1, mforms.HFillFlag | mforms.VFillFlag)
+        info_table.add(self.create_labeled_info("Threads Cached:", "Threads Cached", "lbl_Threads_cached"),                           3, 4, 0, 1, mforms.HFillFlag | mforms.VFillFlag)
+        info_table.add(self.create_labeled_info("Rejected (over limit):", "Rejected", "lbl_Connection_errors_max_connections"), 4, 5, 0, 1, mforms.HFillFlag | mforms.VFillFlag)
 
-        info_table.add(self.create_labeled_info("Total Connections:", "lbl_Connections"),                           0, 1, 1, 2, mforms.HFillFlag)
-        info_table.add(self.create_labeled_info("Connection Limit:", "lbl_max_connections"),                        1, 2, 1, 2, mforms.HFillFlag)
-        info_table.add(self.create_labeled_info("Aborted Clients:", "lbl_Aborted_clients"),                         2, 3, 1, 2, mforms.HFillFlag)
-        info_table.add(self.create_labeled_info("Aborted Connections:", "lbl_Aborted_connects"),                    3, 4, 1, 2, mforms.HFillFlag)
-        info_table.add(self.create_labeled_info("Errors:", "lbl_errors", "tooltip_errors"),                         4, 5, 1, 2, mforms.HFillFlag)
+        info_table.add(self.create_labeled_info("Total Connections:", "Total Connections", "lbl_Connections"),                           0, 1, 1, 2, mforms.HFillFlag | mforms.VFillFlag)
+        info_table.add(self.create_labeled_info("Connection Limit:", "Connection Limit", "lbl_max_connections"),                        1, 2, 1, 2, mforms.HFillFlag | mforms.VFillFlag)
+        info_table.add(self.create_labeled_info("Aborted Clients:", "Aborted Clients", "lbl_Aborted_clients"),                         2, 3, 1, 2, mforms.HFillFlag | mforms.VFillFlag)
+        info_table.add(self.create_labeled_info("Aborted Connections:", "Aborted Connections", "lbl_Aborted_connects"),                    3, 4, 1, 2, mforms.HFillFlag | mforms.VFillFlag)
+        info_table.add(self.create_labeled_info("Errors:", "Errors", "lbl_errors", "tooltip_errors"),                         4, 5, 1, 2, mforms.HFillFlag | mforms.VFillFlag)
 
         self.info_table = info_table
-        self.add(info_table, False, True)
-
-        #self.set_padding(8)
-        self.add(self.connection_box, True, True)
+        uiBox.add(info_table, False, True)
 
+        uiBox.add(self.connection_box, True, True)
 
         box = newBox(True)
         self.button_box = box
-        self.add_end(box, False, True)
-
-        box.set_spacing(12)
+        self.button_box.set_spacing(12)
         
         refresh_button = newButton()
         refresh_button.set_text("Refresh")
-        box.add_end(refresh_button, False, True)
+        self.button_box.add_end(refresh_button, False, True)
         refresh_button.add_clicked_callback(weakcb(self, "refresh"))
 
         self.kill_button = newButton()
         self.kill_button.set_text("Kill Connection(s)")
-        box.add_end(self.kill_button, False, True)
+        self.kill_button.set_name("Kill Connections")
+        self.button_box.add_end(self.kill_button, False, True)
         self.kill_button.add_clicked_callback(weakcb(self, "kill_connection"))
         
         self.killq_button = newButton()
         self.killq_button.set_text("Kill Query(s)")
-        box.add_end(self.killq_button, False, True)
+        self.killq_button.set_name("Kill Queries")
+        self.button_box.add_end(self.killq_button, False, True)
         self.killq_button.add_clicked_callback(weakcb(self, "kill_query"))
         
         refresh_label = newLabel("Refresh Rate:")
-        box.add(refresh_label, False, True)
+        self.button_box.add(refresh_label, False, True)
 
         self._menu = mforms.newContextMenu()
         self._menu.add_will_show_callback(self.menu_will_show)
@@ -423,13 +453,15 @@ class WbAdminConnections(WbAdminBaseTab):
         self.refresh_selector.set_selected(refresh_rate_index)
         self.update_refresh_rate()
         self.refresh_selector.add_changed_callback(weakcb(self, "update_refresh_rate"))
-        box.add(self.refresh_selector, False, True)
+        self.button_box.add(self.refresh_selector, False, True)
+        
 
         self.check_box = newBox(True)
         self.check_box.set_spacing(12)
 
         self.hide_sleep_connections = newCheckBox()
         self.hide_sleep_connections.set_text('Hide sleeping connections')
+        self.hide_sleep_connections.set_name('Hide sleeping connections')
         self.hide_sleep_connections.add_clicked_callback(self.refresh)
         self.hide_sleep_connections.set_tooltip('Remove connections in the Sleeping state from the connection list.')
         self.check_box.add(self.hide_sleep_connections, False, True)
@@ -440,6 +472,7 @@ class WbAdminConnections(WbAdminBaseTab):
             self.hide_background_threads = newCheckBox()
             self.hide_background_threads.set_active(True)
             self.hide_background_threads.set_text('Hide background threads')
+            self.hide_background_threads.set_name('Hide background threads')
             self.hide_background_threads.set_tooltip('Remove background threads (internal server threads) from the connection list.')
             self.hide_background_threads.add_clicked_callback(self.refresh)
             self.check_box.add(self.hide_background_threads, False, True)
@@ -447,6 +480,7 @@ class WbAdminConnections(WbAdminBaseTab):
             self.truncate_info = newCheckBox()
             self.truncate_info.set_active(True)
             self.truncate_info.set_text('Don\'t load full thread info')
+            self.truncate_info.set_name('Do Not Load Thread Info')
             self.truncate_info.set_tooltip('Toggle whether to load the entire query information for all connections or just the first 255 characters.\nEnabling this can have a large impact in busy servers or server executing large INSERTs.')
             self.truncate_info.add_clicked_callback(self.refresh)
             self.check_box.add(self.truncate_info, False, True)
@@ -455,6 +489,7 @@ class WbAdminConnections(WbAdminBaseTab):
             self.extra_info_tab = mforms.newTabView(mforms.TabViewSystemStandard)
             self.extra_info_tab.set_size(350, -1)
             self.extra_info_tab.add_tab_changed_callback(self.extra_tab_changed)
+            self.extra_info_tab.set_name("Details Sidebar")
 
             self.connection_details_scrollarea = mforms.newScrollPanel()
             self.connection_details = ConnectionDetailsPanel(self)
@@ -468,16 +503,19 @@ class WbAdminConnections(WbAdminBaseTab):
                 self.mdl_list_box_scrollarea.add(self.mdl_list_box)
 
                 self.mdl_label = mforms.newLabel('Metadata locks (MDL) protect concurrent access to\nobject metadata (not table row/data locks)')
+                self.mdl_label.set_name("Metadata Info")
                 self.mdl_list_box.add(self.mdl_label, False, True)
 
                 label = mforms.newLabel("\nGranted Locks (and threads waiting on them)")
+                label.set_name("Granted Locks Info")
                 label.set_style(mforms.BoldStyle)
                 self.mdl_list_box.add(label, False, True)
                 label = mforms.newLabel("Locks this connection currently owns and\nconnections that are waiting for them.")
+                label.set_name("Lock Connection Info")
                 label.set_style(mforms.SmallHelpTextStyle)
                 self.mdl_list_box.add(label, False, True)
 
-                self.mdl_list_held = mforms.newTreeNodeView(mforms.TreeAltRowColors)
+                self.mdl_list_held = mforms.newTreeView(mforms.TreeAltRowColors)
                 self.mdl_list_held.add_column(mforms.IconStringColumnType, "Object", 130, False)
                 self.mdl_list_held.add_column(mforms.StringColumnType, "Type", 100, False)
                 self.mdl_list_held.add_column(mforms.StringColumnType, "Duration", 100, False)
@@ -494,12 +532,13 @@ class WbAdminConnections(WbAdminBaseTab):
                 self.mdl_blocked_icon.set_image(mforms.App.get().get_resource_path("message_warning.png"))
                 hbox.add(self.mdl_blocked_icon, False, True)
                 self.mdl_waiting_label = mforms.newLabel("Locks this connection is currently waiting for.")
+                self.mdl_waiting_label.set_name("Lock Connection Info")
                 hbox.add(self.mdl_waiting_label, True, True)
                 self.mdl_list_box.add(hbox, False, True)
                 self.mdl_locks_page = self.extra_info_tab.add_page(self.mdl_list_box_scrollarea, "Locks")
 
             if self.ctrl_be.target_version.is_supported_mysql_version_at_least(5, 6, 0):
-                self.attributes_list = mforms.newTreeNodeView(mforms.TreeFlatList|mforms.TreeAltRowColors)
+                self.attributes_list = mforms.newTreeView(mforms.TreeFlatList|mforms.TreeAltRowColors)
                 self.attributes_list.add_column(mforms.StringColumnType, "Attribute", 150, False)
                 self.attributes_list.add_column(mforms.StringColumnType, "Value", 200, False)
                 self.attributes_list.end_columns()
@@ -512,14 +551,27 @@ class WbAdminConnections(WbAdminBaseTab):
             self.show_extras.set_text('Show Details')
             self.show_extras.add_clicked_callback(self.toggle_extras)
             self.check_box.add_end(self.show_extras, False, True)
-
-        self.add(self.check_box, False, True)
+            self.check_box.set_padding(0, 5, 0, 5)
+            
+        
+        footerBox = mforms.newBox(False)
+        footerBox.add_end(self.check_box, False, True)
+        footerBox.add_end(self.button_box, False, True)
+        
+        self.set_footer(footerBox)
         
         self.resume_layout()
         
         self.connection_selected()
         
+        self.relayout()
+        
         dprint_ex(4, "Leave")
+        
+        # Call refresh to load list at least once
+        self.refresh()
+
+        return uiBox
 
 
     def shutdown(self):
@@ -528,19 +580,21 @@ class WbAdminConnections(WbAdminBaseTab):
             self._refresh_timeout = None
 
         
-    def create_labeled_info(self, lbl_txt, lbl_name, tooltip_name = None):
+    def create_labeled_info(self, lbl_txt, acc_name, lbl_name, tooltip_name = None):
         lbox = newBox(True)
         lbox.set_spacing(5)
         l = mforms.newLabel(lbl_txt)
-        lbox.add(l, False, False)
+        l.set_name(acc_name)
+        lbox.add(l, False, True)
         setattr(self, lbl_name, mforms.newLabel(""))
         l.set_style(mforms.BoldStyle)
-        lbox.add(getattr(self, lbl_name), False, False)
+        lbox.add(getattr(self, lbl_name), False, True)
         if tooltip_name != None:
             i = mforms.newImageBox()
             i.set_image(mforms.App.get().get_resource_path("mini_notice.png"))
             i.set_tooltip("")
-            lbox.add(i, False, False)
+            i.set_name(tooltip_name)
+            lbox.add(i, False, True)
             setattr(self, tooltip_name, i)
 
         return lbox
@@ -584,14 +638,14 @@ class WbAdminConnections(WbAdminBaseTab):
                 if not user_thread:
                     user_thread = len([sel for sel in selected_conn if not sel.get_string(8).startswith('BACKGROUND')]) > 0
 
-                item = self._menu.add_item_with_title("Copy", self.copy_selected, "copy_selected")
-                item = self._menu.add_item_with_title("Copy Info", self.copy_selected_info, "copy_selected_info")
-                item = self._menu.add_item_with_title("Show in Editor", self.edit_selected, "edit_selected")
+                item = self._menu.add_item_with_title("Copy", self.copy_selected, "Copy Selected", "copy_selected")
+                item = self._menu.add_item_with_title("Copy Info", self.copy_selected_info, "Copy Selected Info", "copy_selected_info")
+                item = self._menu.add_item_with_title("Show in Editor", self.edit_selected, "Edit Selected", "edit_selected")
                 item.set_enabled(user_thread)
 
                 if self.ctrl_be.target_version.is_supported_mysql_version_at_least(5, 7):
                     self._menu.add_separator()
-                    item = self._menu.add_item_with_title("Explain for Connection", self.explain_selected, "explain")
+                    item = self._menu.add_item_with_title("Explain for Connection", self.explain_selected, "Explain", "explain")
                     if len(selected_conn) != 1:
                         item.set_enabled(False)
                     else:
@@ -600,7 +654,7 @@ class WbAdminConnections(WbAdminBaseTab):
                             item.set_enabled(False)
 
                 if self.new_processlist():
-                    item = self._menu.add_item_with_title("View Thread Stack", self.view_thread_stack, "view_thread_stack")
+                    item = self._menu.add_item_with_title("View Thread Stack", self.view_thread_stack, "View Thread Stack", "view_thread_stack")
                     if len(selected_conn) != 1:
                         item.set_enabled(False)
                     instr_caption = "Enable Instrumentation for Thread"
@@ -608,17 +662,17 @@ class WbAdminConnections(WbAdminBaseTab):
 
                     if all(start_with_yes):
                         instr_caption = "Disable Instrumentation for Thread"
-                    item = self._menu.add_item_with_title(instr_caption, self.enable_disable_instrumentation, "enable_disable_instrumentation")
+                    item = self._menu.add_item_with_title(instr_caption, self.enable_disable_instrumentation, "Enable Instrumentation", "enable_disable_instrumentation")
                     if any(start_with_yes) and not all(start_with_yes):
                         item.set_enabled(False)                    
 
                 self._menu.add_separator()
-                item = self._menu.add_item_with_title("Kill Query(s)", self.kill_query, "kill_query")
+                item = self._menu.add_item_with_title("Kill Query(s)", self.kill_query, "Kill Query", "kill_query")
                 item.set_enabled(user_thread)
-                item = self._menu.add_item_with_title("Kill Connection(s)", self.kill_connection, "kill_connection")
+                item = self._menu.add_item_with_title("Kill Connection(s)", self.kill_connection, "Kill Connection", "kill_connection")
                 item.set_enabled(user_thread)
                 self._menu.add_separator()
-            self._menu.add_item_with_title("Refresh", self.refresh, "refresh")
+            self._menu.add_item_with_title("Refresh", self.refresh, "Refresh", "refresh")
 
     def column_resized(self, col):
         widths = []
@@ -691,7 +745,7 @@ class WbAdminConnections(WbAdminBaseTab):
                     node = self.attributes_list.add_node()
                     node.set_string(0, result.stringByName("ATTR_NAME"))
                     node.set_string(1, result.stringByName("ATTR_VALUE"))
-        except Exception, e:
+        except Exception as e:
             import traceback
             log_error("Error looking up attribute information: %s\n" % traceback.format_exc())
             mforms.Utilities.show_error("Lookup Connection Attributes", "Error looking up connection attributes: %s" % e, "OK", "", "")
@@ -776,38 +830,13 @@ class WbAdminConnections(WbAdminBaseTab):
                                 subnode.set_string(2, subresult.stringByName("LOCK_DURATION"))
             else:
                 waiting_label_text = ""
-        except Exception, e:
+        except Exception as e:
             import traceback
             log_error("Error looking up metadata lock information: %s\n" % traceback.format_exc())
             mforms.Utilities.show_error("Lookup Metadata Locks", "Error looking up metadata lock information: %s" % e, "OK", "", "")
 
         self.mdl_waiting_label.set_text(waiting_label_text)
 
-
-    def page_activated(self):
-        WbAdminBaseTab.page_activated(self)
-
-        if not self.ui_created:
-            self.create_ui()
-            self.ui_created = True
-        
-        self.page_active = True
-        if self.ctrl_be.is_sql_connected():
-            self.connection_box.show(True)
-            self.info_table.show(True)
-            self.button_box.show(True)
-            self.check_box.show(True)
-        else:
-            self.connection_box.show(False)
-            self.info_table.show(False)
-            self.button_box.show(False)
-            self.check_box.show(False)
-        
-        self.refresh()
-    
-    def page_deactivated(self):
-        self.page_active = False
-    
     def get_process_list(self):
         if self.new_processlist():
             return self.get_process_list_new()
@@ -884,7 +913,7 @@ class WbAdminConnections(WbAdminBaseTab):
             connid = sel.get_long(7)
             try:
                 self.ctrl_be.exec_sql("UPDATE performance_schema.threads SET instrumented = '%s' WHERE thread_id = %d LIMIT 1" % (instr_state, connid))
-            except Exception, e:
+            except Exception as e:
                 log_error("Error enabling thread instrumentation: %s\n" % e)
                 mforms.Utilities.show_error("Toggle Thread Instrumentation", "Error setting instrumentation for thread %d: %s" % (connid, e), "OK",  "", "")
                 break
@@ -899,7 +928,7 @@ class WbAdminConnections(WbAdminBaseTab):
     def enable_mdl_instrumentation(self):
         try:
             self.ctrl_be.exec_sql("UPDATE performance_schema.setup_instruments SET enabled='YES' WHERE name = 'wait/lock/metadata/sql/mdl'")
-        except Exception, e:
+        except Exception as e:
             log_error("Error enabling MDL instrumentation: %s\n" % e)
             mforms.Utilities.show_error("Enable MDL Instrumentation", "Error enabling performance_schema MDL instrumentation.\n%s" % e, "OK",  "", "")
             return
@@ -998,7 +1027,7 @@ class WbAdminConnections(WbAdminBaseTab):
 
 
     def refresh(self, query_result = None, my_serial = 0):
-        if not self.page_active:
+        if not self.page_active():
             dprint_ex(2, "Leave. Page is inactive")
             return True
  
@@ -1042,7 +1071,7 @@ class WbAdminConnections(WbAdminBaseTab):
                     for c, field in enumerate(row):
                         if c in self.long_int_columns:
                             try:
-                                field = long(field)
+                                field = int(field)
                             except Exception:
                                 field = 0
                             r.set_long(c, field)
@@ -1090,7 +1119,7 @@ class WbAdminConnections(WbAdminBaseTab):
 
             try:
                 self.ctrl_be.exec_sql("KILL CONNECTION %s"%connid)
-            except Exception, e:
+            except Exception as e:
                 mforms.Utilities.show_error("Error Killing Connection", "%s" % e, "OK",  "", "")
                 break
                 
@@ -1113,7 +1142,7 @@ class WbAdminConnections(WbAdminBaseTab):
 
             try:
                 self.ctrl_be.exec_sql("KILL QUERY %s"%connid)
-            except Exception, e:
+            except Exception as e:
                 mforms.Utilities.show_error("Error Killing Connection", "Error executing KILL QUERY on thread %d: %s" % (connid, e), "OK",  "", "")
                 break
           
